==> luna_cow-0.6.18.tar.gz: luna_cow-0.6.18/cow/manager/config_loader.py <==
"""Configuration adaptor with support of update events."""

import asyncio
import sys
from configparser import MissingSectionHeaderError
from logging import Logger as BaseLogger
from typing import Awaitable, Callable, Dict, List, Type

from fastjsonschema.exceptions import JsonSchemaException as ValidationError

from ..utils.log import Logger


class Observable:
    """Base class for observables."""

    def __init__(self):
        self.subscribers: List[Awaitable] = []

    def subscribe(self, notify: Awaitable):
        """
        Register callback.

        Args:
            notify: callback
        Raises:
            TypeError in case notify is not awaitable
        """
        if not asyncio.iscoroutinefunction(notify):
            raise TypeError("Pass coroutine")

        self.subscribers.append(notify)

    def notify(self, *args) -> Awaitable:
        """Notify subscribers."""
        waiters = []
        for subscriber in self.subscribers:
            waiters.append(subscriber(*args))
        return asyncio.gather(*waiters)


class ConfigLoader:
    """
    configClass.load() adaptor, stores constant parameters.

    Attributes:
        configClass: A type, config class
        configFile: A str, path to config file
        configuratorAddress: A str, configurator uri
        tags: A List of str, configuration tags
    """

    def __init__(
        self,
        configClass: Type["BasePythonServiceSettingsClass"],
        configFile: str,
        configuratorAddress: str,
        tags: [Dict[str, str]],
    ):
        self.configClass = configClass
        self.configFile = configFile
        self.configuratorAddress = configuratorAddress
        self.tags = tags

    def load(self) -> "BasePythonServiceSettingsClass":
        """Load configuration."""
        try:
            return self.configClass().load(
                configFile=self.configFile,
                configuratorAddress=self.configuratorAddress,
                tags=self.tags,
                asyncLoad=False,
            )
        except (MissingSectionHeaderError, KeyError, ValueError, ValidationError, TypeError) as exc:
            # error occurred while loading the config, so application logger has not been initialized yet
            BaseLogger("ERROR").error(f"Failed to load the config: {exc}", exc_info=False)
            sys.exit(1)

    def asyncLoad(self) -> Awaitable:
        """Load configuration asynchronously."""
        return self.configClass().load(
            configFile=self.configFile, configuratorAddress=self.configuratorAddress, tags=self.tags, asyncLoad=True
        )


class InvalidConfigError(Exception):
    """Exception for notifying base process about invalid config."""


class RetryablePuller(Observable):
    """
    Config puller with retries.

    Attributes:
        configLoader: A ConfigLoader instance
        initialConfig: An instance of config
        maxInvalidAttempts: An int, max number of retries
        invalidAttemptsCount: An int, number of invalid load attempts
        logger: Puller logger
        previousConfig: An instance of config
        connectionValidator: Callable, returns whether services is ok with a new config
    """

    def __init__(
        self,
        configLoader: ConfigLoader,
        initialConfig: "BasePythonServiceSettingsClass",
        maxInvalidAttempts: int,
        connectionValidator: Callable[["BasePythonServiceSettingsClass"], Awaitable[bool]],
    ):
        super().__init__()
        self.configLoader = configLoader
        self.invalidAttemptsCount = 0
        self.maxInvalidAttempts = maxInvalidAttempts
        self.logger = Logger("config puller")
        self.previousConfig = initialConfig
        self.connectionValidator = connectionValidator

    async def pull(self):
        """
        Perform config fetch, validation and notification.

        Raises:
            InvalidConfigError if number of invalid attempts exceeds limit
        """
        try:
            config = await self.configLoader.asyncLoad()
        except Exception as exc:  # pylint: disable-msg=broad-except
            self.logger.warning(f"failed to fetch config: {exc}")
            return

        # Dont do anything if new config is valid and its the same as the previous one
        if config == self.previousConfig and self.invalidAttemptsCount == 0:
            return
        self.logger.info("received new config")
        if not await self.connectionValidator(config):
            self.invalidAttemptsCount += 1
            self.logger.error(f"failed to validate new config, attempt {self.invalidAttemptsCount}")
            if self.invalidAttemptsCount >= self.maxInvalidAttempts:
                raise InvalidConfigError()

        else:
            self.logger.info("notify all subscribers")
            await self.notify(config)
            self.invalidAttemptsCount = 0
        self.previousConfig = config

==> luna_cow-0.8.18.tar.gz: luna_cow-0.8.18/cow/manager/config_loader.py <==
"""Configuration adaptor with support of update events."""

import asyncio
import sys
from configparser import MissingSectionHeaderError
from logging import Logger as BaseLogger
from typing import Awaitable, Callable, Dict, List, Type

from fastjsonschema.exceptions import JsonSchemaException as ValidationError

from ..utils.log import Logger


class Observable:
    """Base class for observables."""

    def __init__(self):
        self.subscribers: List[Awaitable] = []

    def subscribe(self, notify: Awaitable):
        """
        Register callback.

        Args:
            notify: callback
        Raises:
            TypeError in case notify is not awaitable
        """
        if not asyncio.iscoroutinefunction(notify):
            raise TypeError("Pass coroutine")

        self.subscribers.append(notify)

    def notify(self, *args) -> Awaitable:
        """Notify subscribers."""
        waiters = []
        for subscriber in self.subscribers:
            waiters.append(subscriber(*args))
        return asyncio.gather(*waiters)


class ConfigLoader:
    """
    configClass.load() adaptor, stores constant parameters.

    Attributes:
        configClass: A type, config class
        configFile: A str, path to config file
        configuratorAddress: A str, configurator uri
        tags: A List of str, configuration tags
    """

    def __init__(
        self,
        configClass: Type["BasePythonServiceSettingsClass"],
        configFile: str,
        configuratorAddress: str,
        tags: [Dict[str, str]],
    ):
        self.configClass = configClass
        self.configFile = configFile
        self.configuratorAddress = configuratorAddress
        self.tags = tags

    def load(self) -> "BasePythonServiceSettingsClass":
        """Load configuration."""
        try:
            return self.configClass().load(
                configFile=self.configFile,
                configuratorAddress=self.configuratorAddress,
                tags=self.tags,
                asyncLoad=False,
            )
        except (MissingSectionHeaderError, KeyError, ValueError, ValidationError, TypeError) as exc:
            # error occurred while loading the config, so application logger has not been initialized yet
            logger = BaseLogger("ERROR")
            logger.error(f"Failed to load the config: {exc}", exc_info=False)
            logger.error(f"Configurator address: '{self.configuratorAddress}'")
            logger.error(f"Config file path: '{self.configFile}'")
            logger.error(f"Tags: '{self.tags}'")
            sys.exit(1)

    def asyncLoad(self) -> Awaitable:
        """Load configuration asynchronously."""
        return self.configClass().load(
            configFile=self.configFile, configuratorAddress=self.configuratorAddress, tags=self.tags, asyncLoad=True
        )


class InvalidConfigError(Exception):
    """Exception for notifying base process about invalid config."""


class RetryablePuller(Observable):
    """
    Config puller with retries.

    Attributes:
        configLoader: A ConfigLoader instance
        initialConfig: An instance of config
        maxInvalidAttempts: An int, max number of retries
        invalidAttemptsCount: An int, number of invalid load attempts
        logger: Puller logger
        previousConfig: An instance of config
        connectionValidator: Callable, returns whether services is ok with a new config
    """

    def __init__(
        self,
        configLoader: ConfigLoader,
        initialConfig: "BasePythonServiceSettingsClass",
        maxInvalidAttempts: int,
        connectionValidator: Callable[["BasePythonServiceSettingsClass"], Awaitable[bool]],
    ):
        super().__init__()
        self.configLoader = configLoader
        self.invalidAttemptsCount = 0
        self.maxInvalidAttempts = maxInvalidAttempts
        self.logger = Logger("config puller")
        self.previousConfig = initialConfig
        self.connectionValidator = connectionValidator

    async def pull(self):
        """
        Perform config fetch, validation and notification.

        Raises:
            InvalidConfigError if number of invalid attempts exceeds limit
        """
        try:
            config = await self.configLoader.asyncLoad()
        except Exception as exc:  # pylint: disable-msg=broad-except
            self.logger.warning(f"failed to fetch config: {exc}")
            return

        # Dont do anything if new config is valid and its the same as the previous one
        if config == self.previousConfig and self.invalidAttemptsCount == 0:
            return
        self.logger.info("received new config")
        if not await self.connectionValidator(config):
            self.invalidAttemptsCount += 1
            self.logger.error(f"failed to validate new config, attempt {self.invalidAttemptsCount}")
            if self.invalidAttemptsCount >= self.maxInvalidAttempts:
                raise InvalidConfigError()

        else:
            self.logger.info("notify all subscribers")
            await self.notify(config)
            self.invalidAttemptsCount = 0
        self.previousConfig = config
